@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991-2007 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2009 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -19,12 +19,25 @@
    The Quartz way is complicated, and a little flaky, and maybe faster
    but I can't tell.  Define this to do it the Quartz way.
  */
-//#define BE_QUARTZY
+#ifdef USE_IPHONE
+# define BE_QUARTZY
+#endif /* !USE_IPHONE */
+
+#if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4
+  /* In 10.5 and later, NSColor wants CGfloat args, but that type
+     does not exist in 10.4. */
+  typedef float CGFloat;
+#endif
+
 
 @interface DaliClockView (ForwardDeclarations)
+
 + (NSUserDefaultsController *)userDefaultsController;
 + (void)setUserDefaultsController:(NSUserDefaultsController *)ctl;
+
 - (void)setForeground:(NSColor *)fg background:(NSColor *)bg;
+
+
 - (void)clockTick;
 - (void)colorTick;
 - (void)dateTick;
@@ -48,6 +61,29 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
     return;
   initialized_p = YES;
 
+# ifdef USE_IPHONE
+  /* Why doesn't iPhone have NSUserDefaultsController?  WTF. */
+  {
+    controller = [NSUserDefaults standardUserDefaults];
+    NSString *bundle = [[[NSBundle mainBundle] bundlePath]
+                         stringByAppendingPathComponent:
+                           @"Settings.bundle/Root.plist"];
+    NSDictionary *plist = [[NSDictionary dictionaryWithContentsOfFile:bundle]
+                            objectForKey:@"PreferenceSpecifiers"];
+    NSMutableDictionary *defaults = [NSMutableDictionary new];
+    for (NSDictionary *setting in plist) {
+      NSString *key = [setting objectForKey:@"Key"];
+      if (key)
+        [defaults setObject:[setting objectForKey:@"DefaultValue"] 
+                     forKey:key];
+    }
+    // Persist the newly initialized default settings and re-retrieve them
+    [controller setPersistentDomain:defaults
+                            forName:[[NSBundle mainBundle] bundleIdentifier]];
+    controller = [NSUserDefaults standardUserDefaults];
+  }
+# endif /* USE_IPHONE */
+
   staticUserDefaultsController = controller;
   [controller retain];
 
@@ -78,15 +114,25 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
     nil];
 
   NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:100];
+
+# ifndef USE_IPHONE
   [dict addEntriesFromDictionary: [controller initialValues]];
+# endif /* !USE_IPHONE */
   [dict addEntriesFromDictionary:defaults];
+# ifdef USE_IPHONE
+  for (NSString *key in dict) {
+    [controller setObject:[dict objectForKey:key] forKey:key];
+  }
+# else  /* !USE_IPHONE */
   [controller setInitialValues:dict];
   [[controller defaults] registerDefaults:dict];
+# endif /* !USE_IPHONE */
 }
 
 
 - (void)bindPreferences
 {
+# ifndef USE_IPHONE
   NSUserDefaultsController *controller = staticUserDefaultsController;
 
   [self    bind:@"hourStyle"
@@ -124,6 +170,34 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
        toObject:controller
     withKeyPath:@"values.initialBackgroundColor"
         options:colorBindingOptions];
+# else  /* USE_IPHONE */
+
+  // WTF, why don't we have bindings for this!!
+  NSUserDefaultsController *c = [[self class] userDefaultsController];
+  [self setHourStyle:(int)[[c integerForKey:@"hourStyle"] intValue]];
+  [self setTimeStyle:(int)[[c integerForKey:@"timeStyle"] intValue]];
+  [self setDateStyle:(int)[[c integerForKey:@"dateStyle"] intValue]];
+  [self setCycleSpeed:(float)[c floatForKey:@"cycleSpeed"]];
+  [self setUsesCountdownTimer:(int)[c boolForKey:@"usesCountdownTimer"]];
+  [self setCountdownDate:(NSDate *)[c objectForKey:@"countdownDate"]];
+# if 0 // #### OH FOR FUCK'S SAKE
+  [self setInitialForegroundColor:
+          [[c objectForKey:@"initialForegroundColor"] decodeObject]];
+  [self setInitialBackgroundColor:
+          [[c objectForKey:@"initialBackgroundColor"] decodeObject]];
+# else
+  NSColor *deffg = [NSColor  blueColor];
+  NSColor *defbg = [NSColor colorWithCalibratedHue:0.50
+                                        saturation:1.00
+                                        brightness:0.70   // cyan, but darker
+                                             alpha:0.30]; // and translucent
+  [self setInitialForegroundColor:deffg];
+  [self setInitialBackgroundColor:defbg];
+  fg = [deffg retain];
+  bg = [defbg retain];
+# endif
+
+# endif /* USE_IPHONE */
 }
 
 
@@ -134,34 +208,42 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 
   memset (&config, 0, sizeof(config));
 
+# ifndef BE_QUARTZY
   img = [[[NSImage alloc] init] retain];
   if (! img) abort();
+# endif /* !BE_QUARTZY */
 
   config.max_fps = 12;
 
   [self setOwnWindow:YES];  // by default, we may change window background
 
+# ifndef USE_IPHONE
   // Tell the color selector widget to show the "opacity" slider.
   [[NSColorPanel sharedColorPanel] setShowsAlpha:YES];
+# endif /* !USE_IPHONE */
 
   // initialize the fonts and bitmaps
   [self setFrameSize:[self frame].size];
 
+  [self bindPreferences];
+
   [self clockTick];
   [self colorTick];
 
-  [self bindPreferences];
-
   return self;
 }
 
 
 -(oneway void)dealloc
 {
+# ifndef BE_QUARTZY
   [img release];
+# endif /* !BE_QUARTZY */
   [super dealloc];
 }
 
+#ifndef USE_IPHONE
+
 /* Called when the user starts interactively resizing the window.
  */
 - (void)viewWillStartLiveResize
@@ -176,11 +258,14 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
   [self setFrameSize:[self frame].size];
 }
 
+#endif /* !USE_IPHONE */
+
 
 /* Called when the View is resized.
  */
 - (void)setFrameSize:(NSSize)newSize
 {
+# ifndef USE_IPHONE
   [super setFrameSize:newSize];
 
   // If the user is interactively resizing the window, don't regenerate
@@ -188,6 +273,7 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
   // already on the window instead, which reduces flicker when the
   // target bitmap size shifts over a boundary).
   if ([self inLiveResize])  return;
+# endif /* !USE_IPHONE */
 
   int ow = config.width;
   int oh = config.height;
@@ -219,6 +305,8 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 }
 
 
+#ifndef USE_IPHONE
+
 /* Announce our willingness to accept keyboard input.
  */
 - (BOOL)acceptsFirstResponder
@@ -260,26 +348,39 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 FAIL:
   [super keyDown:ev];
 }
+#endif /* !USE_IPHONE */
 
 
 /* This is called from the timer (and other places) to change the colors.
  */
 - (void)setForeground:(NSColor *)new_fg background:(NSColor *)new_bg
 {
-  if (fg != new_fg)
-  {
+  if (fg != new_fg) {
     if (fg) [fg release];
+# ifdef USE_IPHONE
+    fg = [[new_fg copy] retain];
+# else  /* !USE_IPHONE */
     fg = [[new_fg colorUsingColorSpaceName:NSCalibratedRGBColorSpace] retain];
+# endif /* !USE_IPHONE */
   }
-  if (bg != new_bg)
-  {
+  if (bg != new_bg) {
     if (bg) [bg release];
+# ifdef USE_IPHONE
+    bg = [[new_bg copy] retain];
+# else  /* !USE_IPHONE */
     bg = [[new_bg colorUsingColorSpaceName:NSCalibratedRGBColorSpace] retain];
+# endif /* !USE_IPHONE */
   }
+
+# ifdef USE_IPHONE
+    [self setNeedsDisplayInRect:[self frame]];
+# else /* !USE_IPHONE */
   [self setNeedsDisplay:TRUE];
+# endif /* !USE_IPHONE */
 }
 
 
+
 /* The big kahuna refresh method.  Draw the current contents of the bitmap
    onto the window.  Sounds easy, doesn't it?
  */
@@ -400,8 +501,16 @@ FAIL:
 
   float fgr, fgg, fgb, fga;
   float bgr, bgg, bgb, bga;
+# ifndef USE_IPHONE
   [fg getRed:&fgr green:&fgg blue:&fgb alpha:&fga];
   [bg getRed:&bgr green:&bgg blue:&bgb alpha:&bga];
+# else  /* USE_IPHONE */
+  const CGFloat *rgba;
+  rgba = CGColorGetComponents ([fg CGColor]);
+  fgr = rgba[0]; fgg = rgba[1]; fgb = rgba[2]; fga = rgba[3]; 
+  rgba = CGColorGetComponents ([bg CGColor]);
+  bgr = rgba[0]; bgg = rgba[1]; bgb = rgba[2]; bga = rgba[3]; 
+# endif /* USE_IPHONE */
 
   CGDataProviderRef provider =
     CGDataProviderCreateWithData (NULL /* info */, config.bitmap,
@@ -426,7 +535,12 @@ FAIL:
                                 );
   CGDataProviderRelease (provider);
 
+# ifdef USE_IPHONE
+  CGContextRef cgc = UIGraphicsGetCurrentContext();
+# else  /* !USE_IPHONE */
   CGContextRef cgc = [[NSGraphicsContext currentContext] graphicsPort];
+# endif /* !USE_IPHONE */
+
   CGRect bgrect, fgrect;
   bgrect.origin.x    = framerect.origin.x;
   bgrect.origin.y    = framerect.origin.y;
@@ -437,6 +551,7 @@ FAIL:
   fgrect.size.width  = torect.size.width;
   fgrect.size.height = torect.size.height;
 
+
   if (! bgmask) {
     /* foreground is solid; background is solid or transparent.
        we can just blast the background down as a rectangle.
@@ -486,6 +601,11 @@ FAIL:
     CGContextRestoreGState (cgc);
   }
 
+# ifdef USE_IPHONE
+  CGContextScaleCTM (cgc, 1, -1);
+  CGContextTranslateCTM (cgc, 0, -framerect.size.height);
+# endif /* USE_IPHONE */
+
   CGContextSetRGBFillColor (cgc, fgr,fgg,fgb,fga);
   CGContextClipToMask (cgc, fgrect, fgmask);
   CGContextFillRect (cgc, fgrect);
@@ -510,7 +630,7 @@ FAIL:
   unsigned char *scanin  = config.bitmap;
   unsigned char *scanout = pixmap;
 
-  float rf, gf, bf, af;
+  CGFloat rf, gf, bf, af;
   [fg getRed:&rf green:&gf blue:&bf alpha:&af];
   unsigned char fgr = rf * 255.0;
   unsigned char fgg = gf * 255.0;
@@ -541,7 +661,6 @@ FAIL:
     scanin += (config.width + 7) >> 3;
   }
 }
-
 #endif /* !BE_QUARTZY */
 
 
@@ -557,8 +676,11 @@ FAIL:
 
   if (config.max_fps <= 0) abort();
 
+# ifndef USE_IPHONE
   NSWindow *w = [self window];
-  if (w && ![w isMiniaturized]) {   // noop if no window yet, or if iconified.
+  if (w && ![w isMiniaturized])   // noop if no window yet, or if iconified.
+# endif /* !USE_IPHONE */
+  {
     struct timeval now;
     struct timezone tzp;
     gettimeofday (&now, &tzp);
@@ -567,7 +689,12 @@ FAIL:
     [self colorizePixmap];
 # endif /* !BE_QUARTZY */
     // [[self window] invalidateShadow]; // windows with shadows flicker...
+
+# ifdef USE_IPHONE
+    [self setNeedsDisplayInRect:[self frame]];
+# else  /* !USE_IPHONE */
     [self setNeedsDisplay:TRUE];
+# endif /* !USE_IPHONE */
   }
 
   // re-schedule the timer according to current fps.
@@ -595,16 +722,31 @@ FAIL:
 
   if ([self window]) {  // do nothing if no window yet
 
-    float h, s, v, a;
+    CGFloat h, s, v, a;
     NSColor *fg2, *bg2;
     float tick = 1.0 / 360.0;   // cycle H by one degree per tick
 
+# ifdef USE_IPHONE
+    const CGFloat *hsv = CGColorGetComponents([fg CGColor]);
+    h = hsv[0]; s = hsv[1]; v = hsv[2];
+    a = CGColorGetAlpha([fg CGColor]);
+# else  /* !USE_IPHONE */
     [fg getHue:&h saturation:&s brightness:&v alpha:&a];
+# endif /* !USE_IPHONE */
+
     h += tick;
     while (h > 1.0) h -= 1.0;
     fg2 = [NSColor colorWithCalibratedHue:h saturation:s brightness:v alpha:a];
 
+# ifdef USE_IPHONE
+    hsv = CGColorGetComponents([bg CGColor]);
+    h = hsv[0]; s = hsv[1]; v = hsv[2];
+    a = CGColorGetAlpha([bg CGColor]);
+# else  /* !USE_IPHONE */
     [bg getHue:&h saturation:&s brightness:&v alpha:&a];
+# endif /* !USE_IPHONE */
+
+
     h += tick * 0.91;   // cycle bg slightly slower than fg, for randomosity.
     while (h > 1.0) h -= 1.0;
     bg2 = [NSColor colorWithCalibratedHue:h saturation:s brightness:v alpha:a];
@@ -661,6 +803,7 @@ FAIL:
  */
 - (IBAction)aboutClick:(id)sender
 {
+# ifndef USE_IPHONE
   NSBundle *nsb = [NSBundle bundleForClass:[self class]];
   NSAssert1 (nsb, @"no bundle for class %@", [self class]);
 
@@ -691,6 +834,7 @@ FAIL:
 
   [[NSApplication sharedApplication]
     orderFrontStandardAboutPanelWithOptions:dict];
+# endif /* !USE_IPHONE */
 }
 
 
@@ -766,7 +910,9 @@ FAIL:
 {
   if (initialForegroundColor != anInitialForegroundColor) {
     [initialForegroundColor release];
+# ifndef USE_IPHONE
     initialForegroundColor = [anInitialForegroundColor copy];
+# endif /* !USE_IPHONE */
     [self setForeground:initialForegroundColor
              background:initialBackgroundColor];
   }
@@ -781,7 +927,9 @@ FAIL:
 {
   if (initialBackgroundColor != anInitialBackgroundColor) {
     [initialBackgroundColor release];
+# ifndef USE_IPHONE
     initialBackgroundColor = [anInitialBackgroundColor copy];
+# endif /* !USE_IPHONE */
     [self setForeground:initialForegroundColor
              background:initialBackgroundColor];
   }
