@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991-2006 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2008 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -10,9 +10,27 @@
  * implied warranty.
  */
 
-#import <Cocoa/Cocoa.h>
+#ifdef USE_IPHONE
 
-int main(int argc, char *argv[])
+# import <UIKit/UIKit.h>
+
+int
+main (int argc, char *argv[])
 {
-    return NSApplicationMain(argc,  (const char **) argv);
+  NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
+  int ret = UIApplicationMain (argc, argv, nil, nil);
+  [pool release];
+  return ret;
 }
+
+#else /* !USE_IPHONE */
+
+# import <Cocoa/Cocoa.h>
+
+int
+main (int argc, char *argv[])
+{
+  return NSApplicationMain(argc,  (const char **) argv);
+}
+
+#endif /* !USE_IPHONE */
